
var GFX = {
	scene: null,
	canvas: null,
	mouse: null,
	sprite_over: null,
	path: "data/",
	
	offset: vec2.create(),
	
	init: function()
	{
		//create the rendering context
		var canvas = document.createElement("canvas");
		var root = document.querySelector("#game");
		var rect = root.getBoundingClientRect();
		canvas.width = rect.width;
		canvas.height = rect.height;	
		root.appendChild( canvas );	
		
		var context = canvas.getContext("2d");
		this.canvas = context.canvas;
		
		this.loop = new GameLoop( this.canvas );
		this.loop.ondraw = this.ondraw.bind(this);
		this.loop.onupdate = GAME.update.bind( GAME );
		
		GFX.getImage("frame.png");
		
		//user input
		this.loop.onmousedown = function(e)
		{
			if( !GFX.onClick(e) )
				GAME.onmousedown(e);
		}
		
		this.loop.onmousemove = function(e)
		{
			GAME.onmousemove(e);
		}

		this.loop.onmouseup = function(e)
		{
			GAME.onmouseup(e);
		}
		
		this.loop.onmousewheel = function(e)
		{
		}
		
		this.loop.start();
	},
	
	ondraw: function()
	{
		var ctx = this.canvas.getContext("2d");
		ctx.fillStyle = "#544f49";
		ctx.fillRect(0,0, this.canvas.width, this.canvas.height );
		
		ctx.save();
		
		//this.offset[0] = (this.canvas.width - 1024) * 0.5;
		//this.offset[1] = (this.canvas.height - 700) * 0.5;
		ctx.translate( this.offset[0], this.offset[1] );
		
		GFX.renderRoom( ctx );
		
		if( !GAME.current_room.noframe )
			GFX.renderHUD( ctx );
		else if ( GAME.current_room && GAME.current_room.ondraw )
				GAME.current_room.ondraw( ctx );
		
		ctx.restore();
	},
		
	renderRoom: function( ctx, room )
	{
		room = room || GAME.current_room;
		if(!room)
			return;
			
		var mouse = this.loop.mouse;
		var cursor = "";
		this.sprite_over = null;

		if( room.bg_room )
			this.renderRoom( ctx, GAME.rooms[ room.bg_room ] );
		
		if(room.background)
		{
			ctx.fillStyle = room.background;
			ctx.fillRect(0,0, ctx.canvas.width, ctx.canvas.height );
		}
		
		for(var i in room.sprites)
		{
			var sprite = room.sprites[i];
			
			if(sprite.special)
			{
				this.renderSpecial( ctx, sprite, room );
				continue;
			}
			
			var pos = sprite.position || [0,0];
			
			var img_src = null;
			
			if( sprite.alpha !== undefined )
				ctx.globalAlpha = sprite.alpha;
			else
				ctx.globalAlpha = 1;
			
			if(sprite.texture)
				img_src = sprite.texture;
				
			if( sprite.area && !mouse.dragging )
			{
				if( mouse.isInsideRect( pos[0] + this.offset[0], pos[1] + this.offset[1], sprite.area[0], sprite.area[1], true  ) )
				{
					if( sprite.texture_over )
						img_src = sprite.texture_over;
					if(sprite.action)
						cursor = "pointer";
					if(sprite.cursor)
						cursor = sprite.cursor;
					this.sprite_over = sprite;
				}
			}
			
			if(img_src)
			{
				var img = this.getImage( img_src );
				if(img)
				{
					ctx.drawImage( img, pos[0], pos[1] );
				}
			}
		}
		
		ctx.globalAlpha = 1;
		this.canvas.style.cursor = cursor;
	},
	
	renderHUD: function( ctx )
	{
		var canvas = ctx.canvas;
		
		var img = this.getImage( "frame.png" );
		if(img)
			ctx.drawImage( img, 0,0 );
			
		if( GAME.current_room )
		{
			if( GAME.current_room.ondraw )
				GAME.current_room.ondraw( ctx );

			//icons
			if(GAME.current_room.back)
			{
				var ontop = GFX.loop.mouse.isInsideRect(10,10,100,100, true);
				var icon = ontop ? "icon-arrow-over.png" : "icon-arrow.png";
				if(ontop)
					canvas.style.cursor = "pointer";
				var img = this.getImage(icon);
				if(img)
					ctx.drawImage( img, 10, 10 );
			}
			
			if(!GAME.current_room.noframe)
			{
				var ontop = GFX.loop.mouse.isInsideRect(900,450,100,100, true);
				var icon = ontop ? "icon-clock-over.png" : "icon-clock.png";
				if(ontop)
					canvas.style.cursor = "pointer";
				var img = this.getImage(icon);
				if(img)
					ctx.drawImage( img, 900, 450 );
			}			
		}

		//msg
		ctx.font = "18px Arial";
		ctx.fillStyle = "white";
		var time = getTime();
		
		ctx.save();
		ctx.beginPath();
		ctx.rect(0,ctx.canvas.height - 134, ctx.canvas.width, 130 );
		ctx.clip();
		
		var start = Math.max(0, this.logs.length - 5 );
		var x = (canvas.width - 1024) * 0.5 + 30;
		var y = canvas.height - 20;
		
		if( this.logs.length < 5 )
			y -= (5 - this.logs.length) * 20;
		
		
		for(var i = 0; i < 5; ++i)
		{
			var index = this.logs.length - i - 1;
			var msg = this.logs[ index ];
			if(msg === undefined)
				break;
			ctx.globalAlpha = 1.0 - Math.min(1.0, (time - msg.time) * 0.001) * 0.5; 
			ctx.fillText( msg.text, x, y );
			y -= 20;
		}
		
		ctx.globalAlpha = 1;
		ctx.restore();
		
	},
	
	renderSpecial: function( ctx, sprite, room ){
		
		if(sprite.special == "radio_content")
		{
			if(!room.radio.power)
			{
				ctx.fillStyle = "black";
				ctx.globalAlpha = 0.75;
				ctx.fillRect( 0,0, ctx.canvas.width, ctx.canvas.height );
				ctx.globalAlpha = 1;
			}
			
			ctx.save();
			ctx.translate( room.radio.center[0], room.radio.center[1] );
			
			var f = (room.radio.freq - room.radio.range[0]) / (room.radio.range[1] - room.radio.range[0]);
			f = Math.clamp( f, 0, 1 );
			ctx.rotate( f * Math.PI - Math.PI * 0.5 );
			ctx.strokeColor = "black";
			ctx.lineWidth = 4;
			ctx.beginPath();
			ctx.moveTo(0,-100);
			ctx.lineTo(0,-250);
			ctx.stroke();		
			ctx.restore();
		}
	},
	
	imgs: {},
	
	getImage: function( url, on_complete )
	{
		var img = this.imgs[ url ];
		if(img)
			return img;
		img = new Image();
		img.src = this.path + url;
		img.onload = on_complete;
		this.imgs[ url ] = img;
		return img;
	},
	
	last_msg_time: -1,
	logs: [],
	logs_pending: [],
	
	showText: function( sentences, on_complete )
	{
		if(!sentences)
			return;
			
		if( sentences.constructor === String || sentences.constructor === Number )
			sentences = [ sentences ];
			
		this.last_msg_time = getTime(); 
		for(var i in sentences)
		{
			var sentence = sentences[i];
			this.logs.push( { text: sentence, time: getTime() } );
		}
	},
	
	onClick: function()
	{
		var mouse = GFX.loop.mouse;
		
		//back
		if(GAME.current_room && GAME.current_room.back && mouse.isInsideRect(10,10,100,100, true) )
		{
			GAME.setRoom( GAME.current_room.back );
			return true;
		}
		
		if(GAME.current_room && !GAME.current_room.noframe && mouse.isInsideRect(900,450,100,100, true) )
		{
			GAME.setRoom( "TURN" );
			return true;
		}
		
		
		if( mouse.y > 100 )
			return false;
			
		//click below			
		if(this.logs.length > 20)
			this.logs.shift();
	}
};

Math.clamp = function( f, a, b) {
	if(f < a)
		return a;
	if( f < b )
		return f;
	return b;
}




