var DESK = {
	sprites: [
		{
			id:"bg",
			texture: "desk.png",
		},
		{
			id:"radio",
			position: [629,238],
			texture: "desk_radio_on.png"
		},
		{
			id:"radio_outline",
			position: [600,230],
			area: [320,200],
			texture_over: "desk_radio_outline.png",
			action: "doAction('CHANGE_ROOM','RADIO')"
		},
		{
			id:"manual",
			position: [646,464],
			area: [166,100],
			texture_over: "book.png",
			action: "doAction('CHANGE_ROOM','BOOK')"
		},
		/*
		{
			id:"dossier",
			position: [830,28],
			area: [174,217],
			texture_over: "dossier.png",
			action: "doAction('CHANGE_ROOM','DOSSIER')"
		},
		*/
		{
			id:"map",
			visible: false,
			position: [0,0],
			area: [200,300],
			action: "doAction('CHANGE_ROOM','MAP')",
			texture_over: "map_wall_outline.png"
		}			
	]
}

GAME.rooms["DESK"] = DESK;
		
var RADIO = {
	intro_text2: ["This is my long wave radio."],
	back: "DESK",
	background: "black",
	radio: {
		power: true,
		range: [190, 535], //KHz
		center: [510,350],
		freq: 300
	},
	sprites: [
		{
			id:"bg",
			texture: "radio_bg.png",
		},
		{
			special: "radio_content",
		},
		{
			id:"radio",
			texture: "radio.png",
		},
		{
			id:"led",
			position: [697,133],
			texture: "radio_led.png",
			alpha: 0.2
		},
		{
			id:"tunner",
			position: [257,123],
			area: [500,300],
			cursor: "ns-resize"
		},
		{
			id:"micro",
			position: [157,423],
			area: [100,100],
			texture: "icon-talk.png",
			texture_over: "icon-talk-over.png",
			action: "doAction('CHANGE_ROOM','TALK')"
		}
	],
	
	audios: {
		"static": { src: "data/static.mp3", loop: true },
		"mayday": { src: "data/mayday.mp3", loop: true, freq: 370, range: 10, event: { time: 2, action: "doAction('LISTEN_MAYDAY')" } },
		"voices": { src: "data/voices.mp3", loop: true, freq: 240, range: 10, event: { time: 3, msg: "This guys are always talking..." } },
		"song": { src: "data/song.mp3", loop: true, freq: 340, range: 10, event: { time: 4, msg: "I love this song..." } },
		"patrol": { src: "data/morse.mp3", loop: true, restart: true, freq: 510, range: 4, event: { audio_time: 15, action: "doAction('DISCOVER_PATROL')" } }
	},
	
	oninit: function()
	{
		this.radio.freq = 300 + (Math.random()-0.5) * 100;
		//this.audios.morse.freq = 510 + (Math.random()-0.5) * 40;
	
		for(var i in this.audios)
		{
			var info = this.audios[i];
			var audio = new Audio();
			audio.src = info.src;
			audio.volume = info.volume || 0.5;
			audio.loop = info.loop;
			audio.info = info;
			this.audios[i] = audio;
		}
	},
	
	onTurn: function(turn)
	{
		this.last_triggered_event = null;
		this.radio.freq = 300 + (Math.random()-0.5) * 100;
		this.listening_to = null;
		for(var i in this.audios)
		{
			var audio = this.audios[i];
			audio.currentTime = 0;
		}
	},
	
	onFinish: function()
	{
		for(var i in this.audios)
		{
			var audio = this.audios[i];
			audio.pause();
			audio.volume = 0;
		}
	},
	
	onenter: function()
	{
		this.last_triggered_event = null;
		this.audios.static.play();
		for(var i in this.audios)
		{
			var audio = this.audios[i];
			if(audio.volume)
			{
				audio.play();
				audio.volume = 0.5;
			}
		}
	},

	onleave: function()
	{
		for(var i in this.audios)
		{
			var audio = this.audios[i];
			//audio.pause();
			audio.volume *= 0.2;
		}
	},
	
	onmousedown: function(e){
		if(GAME.mousedown_sprite && GAME.mousedown_sprite.id == "tunner" )
			this.tunning = true;
	},
	
	onmousemove: function(e){
		if(e.dragging)
		{
			var delta = e.deltay - e.deltax;
			this.radio.freq = Math.clamp( this.radio.freq - delta * 0.2 , this.radio.range[0], this.radio.range[1] );
		}
	},
	
	onmouseup: function(e)
	{
		this.tunning = false;
	},
	
	onupdate: function(dt)
	{
		var total = 0;
		for(var i in this.audios)
		{
			var audio = this.audios[i];
			var audio_freq = audio.info.freq;
			var audio_range = audio.info.range || 10;
			
			if(!audio_freq)
				continue;
						
			var f = Math.max(0, 1 - Math.abs(this.radio.freq - audio_freq ) / audio_range );
			audio.volume = f;
			if(f > 0)
				audio.play();
			else
				audio.pause();
			total += f;
			if(f > 0.6)
			{
				if ( this.listening_to != audio )
				{
					this.listening_to = audio;
					this.listening_time = 0;
				}
				else
				{
					var old_time = this.listening_time;
					this.listening_time += dt;
					
					if( audio.info.event )
					{
						var event = audio.info.event;
						if( this.last_triggered_event != event && 
							((event.time && old_time < event.time && this.listening_time > event.time) ||
						    ( event.audio_time && audio.currentTime > event.audio_time) )
						  )
						{
							
							this.last_triggered_event = event;
							if(event.msg)
								GAME.onOperatorSays( event.msg, 2 );
							else if(event.action)
								eval(event.action);
						}
					}
				}
			}
		}

		//led
		this.sprites[3].alpha = Math.clamp( total + Math.random() * 0.2 - 0.2, 0, 1 ); 		
		
		var audio_static = this.audios["static"];
		audio_static.volume = Math.max( 1.0 - total, 0 );
	},

	ondraw: function(ctx)
	{
		if(this.radio.power)
		{
			ctx.font = "20px Arial";
			ctx.fillStyle = "white";
			ctx.fillText( this.radio.freq.toFixed(1) + "KHz", 460,50 );
		}
	},
	
	getSignalWith: function( audio_name )
	{
		var audio = this.audios[ audio_name ];
		if(!audio)
			return 0;
		var audio_freq = audio.info.freq;
		var audio_range = audio.info.range || 10;
		if(!audio_freq)
			return;
		return Math.max(0, 1 - Math.abs(this.radio.freq - audio_freq ) / audio_range );
	}
}

GAME.rooms["RADIO"] = RADIO;
		
		
		
		
		
var MAP = {
	intro_text2: ["This is the map of the area."],
	background: "black",
	back: "DESK",
	sprites: [
		{
			id:"bg",
			position:[-786,-503],
			texture: "map.jpg",
		},
		{
			id:"debug",
			position: [920,0],
			area: [80,40],
			texture_over: "icon-arrow.png",
			action: "doAction('CHANGE_ROOM','BOAT')"
		}
	],
	onmousemove: function(e)
	{
		GFX.canvas.style.cursor = "all-scroll";
		if(e.dragging)
		{
			var map = this.sprites[0];
			map.position[0] = Math.clamp( map.position[0] + e.deltax, -1000, 100);
			map.position[1] = Math.clamp( map.position[1] + e.deltay, -500, 100);
		}
	}
}

GAME.rooms["MAP"] = MAP;


var TALK = {
	back: "RADIO",
	sprites: [],
	bg_room: "RADIO",

	messages: [
		{ text: "REPORT", order: "REPORT" },
		{ text: "WAIT", order: "WAIT" },
		{ text: "GO NORTH", order: "N" },
		{ text: "GO SOUTH", order: "S" },
		{ text: "GO EAST", order: "E" },
		{ text: "GO WEST", order: "WAIT" },
		{ text: "exit", order: "EXIT" }
		/*
		{ text: "GO TO...", options: [
			{ text: "NORTH" },
			{ text: "SOUTH" },
			{ text: "EAST" },
			{ text: "WEST" }
			] }
		*/
	],

	current: null,

	onenter: function()
	{
	
	},

	ondraw: function(ctx)
	{
		ctx.strokeStyle = "white";
		ctx.fillStyle = "black";
		ctx.fillRect( 100,100, 800, 500 );
		ctx.strokeRect( 100,100, 800, 500 );
		ctx.fillStyle = "white";
		ctx.font = "26px Arial";
		ctx.fillText("Which message do want to say?",160,150);
		var y = 200;
		var mouse = GFX.loop.mouse;
		var messages = this.current || this.messages;
		for(var i in messages)
		{
			var msg = messages[i];
			ctx.fillText(msg.text,210,y);

			if(mouse.isInsideRect( 190, y - 40, 200, 40, true ))
			{
				this.over_msg = msg;
				ctx.fillStyle = "cyan";
				ctx.fillRect( 190 - 20, y - 20, 20, 20 );
				ctx.fillStyle = "white";
			}

			y+= 40;
		}
		ctx.canvas.style.cursor = "pointer";
	},

	onmousedown: function(e)
	{
		var msg = this.over_msg;
		if( !msg || msg.order == "exit" )
			return GAME.setRoom("RADIO");

		console.log( msg.text );
		GAME.onOperatorSendOrder( msg.order ); 
		GAME.setRoom("RADIO");
	}
};

GAME.rooms["TALK"] = TALK;


var BOOK = {
	background: "black",
	back: "DESK",
	sprites: [
		{
			id:"bg",
			texture: "manual.jpg",
		}			
	]
}

GAME.rooms["BOOK"] = BOOK;

var DOSSIER = {
	background: "black",
	back: "DESK",
	sprites: []
}

GAME.rooms["DOSSIER"] = DOSSIER;

var INTRO = {
	background: "black",
	noframe: true,
	sprites: [
		{
			id:"bg",
			texture: "intro.png",
			area: [1000,1000],
			action: "doAction('CHANGE_ROOM','DESK')"
		}
	],
	
	ondraw: function(ctx)
	{
		var alpha = 2.0 - GAME.room_time;
		if(alpha > 0)
		{
			ctx.globalAlpha = alpha;
			ctx.fillStyle = "black";
			ctx.fillRect(0,0,ctx.canvas.width, ctx.canvas.height);
			ctx.globalAlpha = 1;
		}
	}
}

GAME.rooms["INTRO"] = INTRO;

var TURN = {
	background: "black",
	noframe: true,
	sprites: [
		{
			id:"bg",
			texture: "hourlater.jpg",
		}
	],
	
	ondraw: function(ctx)
	{
		var alpha = 1.0 - Math.sin( GAME.room_time * Math.PI / 4 );
		if(alpha > 0)
		{
			ctx.globalAlpha = alpha;
			ctx.fillStyle = "black";
			ctx.fillRect(0,0,ctx.canvas.width, ctx.canvas.height);
			ctx.globalAlpha = 1;

		}
		if( GAME.room_time > 4 )
			doAction('DO_TURN');
	}
}

GAME.rooms["TURN"] = TURN;


var END = {
	background: "black",
	noframe: true,
	ending: "out",
	sprites: [
		{
			id:"bg",
			texture: "ending_out.jpg"
		}
	],
	
	onmousedown: function(e)
	{
		if( GAME.room_time > 2 )
			doAction('START');
	},
	
	onupdate: function()
	{
		//if( GAME.room_time > 5 )
		//	doAction('START');
		this.sprites[0].texture = "ending_" + this.ending.toLowerCase() + ".jpg";
	}
}

GAME.rooms["END"] = END;








